"""
level 0: site
level 1 :subject
level 2 :visit
level 3 :form
level 4 :ig
level 5 :item (variable, item_id, question)
"""


class Node:

    def __init__(self, tree, parent_idx=None, level=0):
        self.children_idx: set[int] = set()
        self.parent_idx = parent_idx
        self.values = dict()
        self.level = level
        self.tree = tree

    def add_child(self, index):
        self.children_idx.add(index)

    def has_child(self) -> bool:
        return len(self.children_idx) != 0

    def has_parent(self) -> bool:
        return self.parent_idx is not None

    def set(self, key, value):
        self.values[key] = value

    def get(self, key):
        return self.values.get(key)

    '''
    搜寻特定的子节点
    '''

    def find_child(self, **kwargs):
        if not self.has_child():
            return None
        for i in self.children_idx:
            node = self.tree.nodes[self.level+1][i]
            if node.satisfy(**kwargs):
                return node
        return None

    def get_child(self, index):
        if index in self.children_idx:
            return self.tree.nodes[self.level+1][index]
        return None

    def get_parent(self):
        if not self.has_parent():
            return None
        return self.tree.nodes[self.level-1][self.parent_idx]

    """
    判断两个节点是否相同
    """
    def eql(self, node):
        return self.satisfy(**node.values)

    '''
    根据传入的参数来判断该节点是否满足条件
    '''
    def satisfy(self, **kwargs):
        for k, v in kwargs.items():
            if self.values[k] != v:
                return False
        return True



import cjen
from cjen import BigTangerine
from cjen.sco.scenario import Scenario
from cjen.tests.aiai.test_http import MockService
from cjen.tests.step.ok_response import OKResponse
from cjen.tests.step.ok_step import OkStep
from cjen.tests.step.other_step import OtherStep


class StepMockService(BigTangerine):
    @cjen.http.base_url(uri="http://127.0.0.1:5000")
    def __init__(self):
        super().__init__()

    @cjen.headers.contentType(value="application/json")
    @cjen.http.post_mapping(uri="post_method_json", json_clazz=OKResponse)
    @cjen.step.call(stepName=OkStep.Name, argName="ok")
    @cjen.step.call(stepName=OtherStep.Name, argName="ok")
    def post_method_json(self, *, data, ok: OKResponse = None, resp=None, **kwargs): ...


def test_one_step():
    mock = StepMockService()
    OkStep(mock, None).run()
    OkStep(mock, None).run()
    # mock.post_method_json(data=dict(username="xx", pwd="yyy"))


def test_two_step():
    mock = StepMockService()
    OkStep(mock, None).run()
    OtherStep(mock, None).run()


# 验证在没有执行之前，Step 是不会注册进入Service的
def test_register():
    mock = StepMockService()
    steps = [OkStep(mock, None), OtherStep(mock, None)]
    for i, step in enumerate(steps):
        step.run()
        assert len(mock.step_definitions) == i + 1


def test_new_scenario():
    scenario = Scenario("", MockService())
    scenario.add_step(OkStep)
    scenario.add_step(OtherStep)
    scenario.run()


def test_compatible_old_scenario():
    scenario = Scenario("")
    service = MockService()
    scenario.append_step(OkStep, service)
    scenario.append_step(OtherStep, service)
    scenario.run()


import pytest

from cjen.exceptions import PropertyRequiredException, PropertyRangerException, RequestParamErr
from cjen.nene.properties import required, required_if, should_be_in


class TestObj:
    def __init__(self, data: dict):
        self.data = data

    @property
    @required()
    def name(self): return self.data.get("name")

    @property
    @required_if(proper="name", wish=["jia"])
    def nick_name(self): return self.data.get("nick_name")

    @property
    @should_be_in(limits=["a", "b", "c"])
    def email(self): return "a"

    @property
    @should_be_in(limits=["a", "b", "c"])
    def invalid_email(self): return "A"

    @property
    @should_be_in(limits=["a", "b", "c"], ignore_case=True)
    def ignore_case_email(self): return "A"

    @property
    @required(exception_class=RequestParamErr, err_msg="test")
    def sid(self):
        return None

    @property
    @required_if(proper="name", wish=["jia"], exception_class=RequestParamErr, err_msg="test")
    def fly(self): return None

    @property
    @should_be_in(limits=["a", "b", "c"], ignore_case=True, exception_class=RequestParamErr, err_msg="test")
    def tree(self): return 'd'


def test_required():
    obj = TestObj(dict(name="1"))
    assert obj.name == "1"


def test_required_exception():
    obj = TestObj(dict(name1="1"))
    with pytest.raises(PropertyRequiredException, match='name 必填'):
        obj.name


def test_required_if():
    obj = TestObj(dict(name="1", nick_name='3'))
    assert obj.nick_name == '3'

    obj = TestObj(dict(name="1"))
    assert obj.nick_name is None


def test_required_if_exception():
    obj = TestObj(dict(name="jia"))
    with pytest.raises(PropertyRequiredException, match='nick_name 必填'):
        obj.nick_name


def test_email():
    obj = TestObj(dict(name="jia"))
    assert obj.email == "a"


def test_invalid_email():
    obj = TestObj(dict(name="jia"))
    with pytest.raises(PropertyRangerException):
        obj.invalid_email


def test_ignore_case_email():
    obj = TestObj(dict(name="jia"))
    assert obj.ignore_case_email == 'A'


def test_custom_invalid():
    obj = TestObj(dict(name="jia"))
    with pytest.raises(RequestParamErr):
        obj.sid

    with pytest.raises(RequestParamErr):
        obj.fly
    with pytest.raises(RequestParamErr):
        obj.tree


from .bigtangerine import BigTangerine
from cjen.nene.database_pool import DatabasePool
from .nene.database_info import DataBaseInfo


class DBBigTangerine(BigTangerine):
    def __init__(self, database_info):
        super().__init__()
        self.database_info = DataBaseInfo.factory(database_info )if type(database_info) == dict else database_info
        self.context["cursor"] = DatabasePool.cursor(host=self.database_info.host,
                                                     port=self.database_info.port,
                                                     user=self.database_info.user,
                                                     pwd=self.database_info.pwd,
                                                     database=self.database_info.database)



from cjen.nene.properties import required


class DataBaseInfo:
    def __init__(self, info: dict):
        self.info = info

    @property
    @required()
    def user(self): return self.info.get("user")

    @property
    @required()
    def host(self): return self.info.get("host")

    @property
    @required()
    def pwd(self): return self.info.get("pwd")

    @property
    @required()
    def database(self): return self.info.get("database")

    """
    为了兼容以前传递的字典
    """
    def get(self, key):
        if key == "user": return self.user
        elif key == "host": return self.host
        elif key == "pwd": return self.pwd
        elif key == "database": return self.database

    @classmethod
    def factory(cls, data_dict: dict):
        return DataBaseInfo(data_dict)









from openpyxl import load_workbook


def reader(*, fpath: str = None, header: bool = True, sheet_name: str = None, cols: list = None):
    """
    使用条件：任务对象和函数上都可以添加
    函数参数需带 xlsx_data
    eg:
    @reader .....
    def read(xlsx_data=None): ...
        fpath: xlsx文件的绝对路径
        header: 是否带有列名，默认是有的
        sheet_name: 要带去的 Sheet 名， 不指定的话默认读取第一个
        cols: 期望读取的列
            [1,2,3] 通过序号获取
            ["id","name","desc"] 通过列名获取
        @reader(fpath="", header=True, sheet_name="result", cols=["id","name"])
        返回字典列表: [{},{},{}]
        @reader(fpath="", header=True, sheet_name="result", cols=[0,1])
        返回2维列表: [[],[],[]]
        @reader(fpath="", header=False, sheet_name="result")
        返回2维列表，所有的数据: [[],[],[]]
        @reader(fpath="", header=True, sheet_name="result")
        返回2维列表，所有的数据: [[],[],[]]
    """
    if cols is None: cols = []

    def __wrapper__(func):
        def __inner__(*args, **kwargs):
            xls_path = fpath if fpath is not None else kwargs.get("fpath")
            work_book = load_workbook(filename=xls_path)
            work_sheet = work_book.worksheets[0] if not sheet_name else \
                list(filter(lambda s: s.title == sheet_name, work_book.worksheets))[0]
            cells = [[cell.value for cell in row] for row in work_sheet.rows]
            if not header and not cols: return cells
            if header:
                if not cols: return cells
                if type(cols[0]) == int:
                    kwargs["xlsx_data"] = [[cell[col] for col in cols] for cell in cells]
                    func(*args, **kwargs)
                if type(cols[0]) == str:
                    header_dict = {}
                    for col in cols:
                        header_dict[col] = cells[0].index(col)
                    res = []
                    for cell in cells[1:]:
                        temp = {}
                        for key, index in header_dict.items():
                            temp[key] = cell[index]
                        res.append(temp)
                    kwargs["xlsx_data"] = res
                    func(*args, **kwargs)

        return __inner__

    return __wrapper__



from cjen.exceptions import PropertyRangerException, PropertyRequiredException


def required(exception_class=None, err_msg=None):
    """
    属性为必填
    :param exception_class: 抛出自定义的异常
    :param err_msg: 抛出自定义的异常
    :return:
    """

    def __inner__(func):
        def __wrapper__(*args, **kwargs):
            if func(*args, **kwargs) is None:
                msg = f'{func.__name__} 必填' if not err_msg else err_msg
                exception = PropertyRequiredException(msg) if not exception_class else exception_class(msg)
                raise exception
            return func(*args, **kwargs)

        return __wrapper__

    return __inner__


def required_if(proper: str, wish: list, exception_class=None, err_msg=None):
    """
    如果 proper的值 == wish
    则 本属性必填
    :param proper:
    :param wish:
    :param exception_class: 抛出自定义的异常
    :param err_msg
    :return:
    """

    def __inner__(func):
        def __wrapper__(*args, **kwargs):
            if getattr(args[0], proper) in wish:
                if not func(*args, **kwargs):
                    msg = f'{func.__name__} 必填' if not err_msg else err_msg
                    exception = PropertyRequiredException(msg) if not exception_class else exception_class(msg)
                    raise exception
            return func(*args, **kwargs)

        return __wrapper__

    return __inner__


def should_be_in(limits: list[str], ignore_case=False, exception_class=None, err_msg=None):
    """
    字符串应该在范围内
    :param limits:
    :param ignore_case: 是否忽略大小写敏感，默认是敏感
    :param exception_class: 抛出自定义的异常
    :param err_msg: 抛出自定义的异常
    :return:
    """

    def __inner__(func):
        def __wrapper__(*args, **kwargs):
            data = func(*args, **kwargs)
            if not ignore_case:
                if data not in limits:
                    msg = f'{func.__name__} should in {limits}' if not err_msg else err_msg
                    exception = PropertyRangerException(msg) if not exception_class else exception_class(msg)
                    raise exception

            else:
                if data.lower() not in [char.lower() for char in limits]:
                    msg = f'{func.__name__} should in {limits}' if not err_msg else err_msg
                    exception = PropertyRangerException(msg) if not exception_class else exception_class(msg)
                    raise exception
            return data

        return __wrapper__

    return __inner__



import pytest

from cjen.nene.collections.tree import Tree


@pytest.fixture
def data_one():
    return [dict(site_id=1, site_name="D01")]


def test_one_level_tree(data_one):
    tree = Tree.factory(total_level=1, data=data_one)
    assert len(tree.nodes) == 1
    assert len(tree.get_level_nodes(0)) == 1
    root0 = tree.get_level_nodes(0)[0]
    assert root0.values["site_id"] == 1
    assert root0.values["site_name"] == "D01"
    tree.add(data_one)
    assert len(tree.nodes) == 1
    assert len(tree.get_level_nodes(0)) == 1
    assert root0.values["site_id"] == 1
    assert root0.values["site_name"] == "D01"


@pytest.fixture
def data_two():
    return [[dict(site_id=1, site_name="D01"), dict(subject_id=1, subject_name="S01")],
            [dict(site_id=1, site_name="D01"), dict(subject_id=1, subject_name="S01")],
            [dict(site_id=1, site_name="D01"), dict(subject_id=2, subject_name="S02")],
            [dict(site_id=2, site_name="D02"), dict(subject_id=1, subject_name="S01")],
            [dict(site_id=2, site_name="D02"), dict(subject_id=2, subject_name="S02")],
            [dict(site_id=2, site_name="D02"), dict(subject_id=3, subject_name="S04")],
            [dict(site_id=2, site_name="D03"), dict(subject_id=3, subject_name="S04")]]


def test_two_level_tree(data_two):
    tree = Tree.factory(total_level=2)
    for e in data_two:
        tree.add(e)
        assert len(tree.nodes) == 2

    assert len(tree.get_level_nodes(0)) == 3
    root0 = tree.get_level_nodes(0)[0]
    root1 = tree.get_level_nodes(0)[1]
    root2 = tree.get_level_nodes(0)[2]
    assert root0.values["site_id"] == 1
    assert root0.values["site_name"] == "D01"
    assert root1.values["site_id"] == 2
    assert root1.values["site_name"] == "D02"
    assert root2.values["site_id"] == 2
    assert root2.values["site_name"] == "D03"
    assert len(tree.get_level_nodes(1)) == 6
    assert len(root0.children_idx) == 2
    assert root0.find_child(subject_id=1, subject_name="S01") is not None
    assert root0.find_child(subject_id=1, subject_name="S01").get_parent().satisfy(site_id=1, site_name="D01") is True
    assert root0.find_child(subject_id=2, subject_name="S02") is not None
    assert root0.find_child(subject_id=2, subject_name="S02").get_parent().satisfy(site_id=1, site_name="D01") is True
    assert len(root1.children_idx) == 3
    assert root1.find_child(subject_id=1, subject_name="S01") is not None
    assert root1.find_child(subject_id=1, subject_name="S01").get_parent().satisfy(site_id=2, site_name="D02") is True
    assert root1.find_child(subject_id=2, subject_name="S02") is not None
    assert root1.find_child(subject_id=2, subject_name="S02").get_parent().satisfy(site_id=2, site_name="D02") is True
    assert root1.find_child(subject_id=3, subject_name="S04") is not None
    assert root1.find_child(subject_id=3, subject_name="S04").get_parent().satisfy(site_id=2, site_name="D02") is True
    assert len(root2.children_idx) == 1
    assert root2.find_child(subject_id=3, subject_name="S04") is not None
    assert root2.find_child(subject_id=3, subject_name="S04").get_parent().satisfy(site_id=2, site_name="D03") is True


@pytest.fixture
def data_three():
    return [[dict(site_id=1, site_name="D01"), dict(subject_id=1, subject_name="S01"), dict(visit_id=1, visit_name="V01")],
            [dict(site_id=1, site_name="D01"), dict(subject_id=1, subject_name="S01"), dict(visit_id=1, visit_name="V01")],
            [dict(site_id=1, site_name="D01"), dict(subject_id=1, subject_name="S01"), dict(visit_id=2, visit_name="V02")],
            [dict(site_id=1, site_name="D01"), dict(subject_id=2, subject_name="S02"), dict(visit_id=1, visit_name="V01")],
            [dict(site_id=2, site_name="D02"), dict(subject_id=1, subject_name="S01"), dict(visit_id=1, visit_name="V01")],
            [dict(site_id=2, site_name="D02"), dict(subject_id=2, subject_name="S02"), dict(visit_id=2, visit_name="V02")],
            [dict(site_id=2, site_name="D02"), dict(subject_id=3, subject_name="S04"), dict(visit_id=3, visit_name="V03")],
            [dict(site_id=2, site_name="D03"), dict(subject_id=3, subject_name="S04"), dict(visit_id=1, visit_name="V01")]]


def test_three_level_tree(data_three):
    tree = Tree.factory(total_level=3)
    for e in data_three:
        tree.add(e)
    assert len(tree.nodes) == 3
    tree.refresh(data_three)
    assert len(tree.nodes) == 3
    assert len(tree.get_level_nodes(0)) == 3
    root0 = tree.get_level_nodes(0)[0]
    root1 = tree.get_level_nodes(0)[1]
    root2 = tree.get_level_nodes(0)[2]
    assert root0.values["site_id"] == 1
    assert root0.values["site_name"] == "D01"
    assert root1.values["site_id"] == 2
    assert root1.values["site_name"] == "D02"
    assert root2.values["site_id"] == 2
    assert root2.values["site_name"] == "D03"
    assert len(tree.get_level_nodes(1)) == 6
    assert len(root0.children_idx) == 2
    assert root0.find_child(subject_id=1, subject_name="S01") is not None
    assert root0.find_child(subject_id=1, subject_name="S01").get_parent().satisfy(site_id=1, site_name="D01") is True
    assert root0.find_child(subject_id=2, subject_name="S02") is not None
    assert root0.find_child(subject_id=2, subject_name="S02").get_parent().satisfy(site_id=1, site_name="D01") is True
    assert len(root1.children_idx) == 3
    assert root1.find_child(subject_id=1, subject_name="S01") is not None
    assert root1.find_child(subject_id=1, subject_name="S01").get_parent().satisfy(site_id=2, site_name="D02") is True
    assert root1.find_child(subject_id=2, subject_name="S02") is not None
    assert root1.find_child(subject_id=2, subject_name="S02").get_parent().satisfy(site_id=2, site_name="D02") is True
    assert root1.find_child(subject_id=3, subject_name="S04") is not None
    assert root1.find_child(subject_id=3, subject_name="S04").get_parent().satisfy(site_id=2, site_name="D02") is True
    assert len(root2.children_idx) == 1
    assert root2.find_child(subject_id=3, subject_name="S04") is not None
    assert root2.find_child(subject_id=3, subject_name="S04").get_parent().satisfy(site_id=2, site_name="D03") is True

    assert len(tree.get_level_nodes(2)) == 7
    assert len(root0.find_child(subject_id=1, subject_name="S01").children_idx) == 2
    assert root0.find_child(subject_id=1, subject_name="S01").find_child(visit_id=1, visit_name="V01").get_parent().get_parent().satisfy(site_id=1, site_name="D01") is True
    assert root0.find_child(subject_id=1, subject_name="S01").find_child(visit_id=2, visit_name="V02").get_parent().get_parent().satisfy(site_id=1, site_name="D01") is True
    assert len(root0.find_child(subject_id=2, subject_name="S02").children_idx) == 1
    assert root0.find_child(subject_id=2, subject_name="S02").find_child(visit_id=1, visit_name="V01").get_parent().get_parent().satisfy(site_id=1, site_name="D01") is True
    assert len(root1.find_child(subject_id=1, subject_name="S01").children_idx) == 1
    assert root1.find_child(subject_id=1, subject_name="S01").find_child(visit_id=1, visit_name="V01").get_parent().get_parent().satisfy(site_id=2, site_name="D02") is True
    assert len(root1.find_child(subject_id=2, subject_name="S02").children_idx) == 1
    assert root1.find_child(subject_id=2, subject_name="S02").find_child(visit_id=2, visit_name="V02").get_parent().get_parent().satisfy(site_id=2, site_name="D02") is True
    assert len(root1.find_child(subject_id=3, subject_name="S04").children_idx) == 1
    assert root1.find_child(subject_id=3, subject_name="S04").find_child(visit_id=3, visit_name="V03").get_parent().get_parent().satisfy(site_id=2, site_name="D02") is True
    assert len(root2.find_child(subject_id=3, subject_name="S04").children_idx) == 1
    assert root2.find_child(subject_id=3, subject_name="S04").find_child(visit_id=1, visit_name="V01").get_parent().get_parent().satisfy(site_id=2, site_name="D03") is True



import os
import json


def Tester(Clazzes=[]):
    """
    设置 该装饰器后，Scenario下的step会执行带有Testing标签的测试用例。
    """

    def __wrapper__(func):
        def __inner__(self, *args, **kwargs):
            func(self, *args, **kwargs)
            self.is_run_test = True
            for clazz in Clazzes:
                self.register_test_classes.add(clazz)

        return __inner__

    return __wrapper__


class Scenario:
    # 为了兼容已有的代码
    # 所以增加的参数service 默认为None
    def __init__(self, scenario_dir, service=None):
        self.scenario_dir = scenario_dir
        self.data = dict()
        self.__load()
        self.steps = []
        self.service = service
        self.is_run_test = False
        self.register_test_classes = set()

    def __load(self):
        for root, dirs, files in os.walk(self.scenario_dir):
            for filename in files:
                if filename.endswith(".json"):
                    with open(os.path.join(root, filename), 'r', encoding="UTF-8") as f:
                        self.data[os.path.splitext(filename)[0]] = json.loads(f.read())

    def append_sub_scenario(self, sub_scenario):
        self.steps.append(sub_scenario)

    # Deprecated
    # 为了兼容已有的代码
    # 所以增加了self.service is None 的判断
    def append_step(self, step_class, service):
        if self.service is None:
            self.service = service
        self.steps.append(step_class(self.service, self))

    def add_step(self, step_class):
        self.steps.append(step_class(self.service, self))

    def run(self):
        for step in self.steps:
            step.run()

    def get(self, key):
        return self.data.get(key)


if __name__ == '__main__':
    scenario = Scenario("D:\\github\\eclinical\\eclinical\\standard\\scenarios\\new_study")
    print(scenario.data)



class CaculateUtils:
    @staticmethod
    def total_pip(total: int, percents=[]):
        values =[int(total * percent) for percent in percents]
        for value in values:
            print(value)


if __name__ == '__main__':
    stage = {
        "SIT 冒烟":0.13 ,
        "SIT 第一轮": 0.2,
        "SIT 第二轮": 0.15,
        "SIT 第三轮":  0.07,
        "UAT 第一轮": 0.11,
        "UAT 第二轮": 0.10,
        "UAT 第三轮": 0.11,
        "UAT 交叉测试": 0.12,
        "验收测试": 0.01,
    }
    print(sum(stage.values()))
    CaculateUtils.total_pip(63, list(stage.values()))




from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep


class OkStep(StandardStep):
    Name = "ok_step.py"

    def __init__(self, service, scenario: Scenario):
        super().__init__(service, scenario)

    def _pre_processor(self): ...

    def call_back(self, **kwargs): assert kwargs.get("ok").procCode() == 200

    def _execute(self):
        super()._execute()
        self.service.post_method_json(data=dict(username="xx", pwd="yyy"))





"""
通用的数据结构
目标：把列表字典转换成树形结构
新生成树:
data = [
    [dict(id=1),dict(id=2)]
    [dict(id=1),dict(id=3)]
    [dict(id=1),dict(id=4)]
]
tree = Tree.factory(data) 
        1       
    2   3   4
添加节点:
data = [dict(id=1),dict(id=5)]
tree.add(data)
        1
    2   3   4   5
刷新树:
data = [
    [dict(id=100),dict(id=200)]
    [dict(id=100),dict(id=300)]
    [dict(id=100),dict(id=400)]
]
tree.refresh(data)
        100
    200   300   400
"""
from cjen.nene.collections.node import Node


class Tree:
    """
    total_level
    设置树形结构的层数
    """
    def __init__(self, total_level):
        self.total_level = total_level
        self.nodes: list[list[Node]] = self.new_node_list()

    def new_node_list(self):
        return [[] for i in range(self.total_level)]

    def __create_node(self, parent_idx=None, level=0, **kwargs):
        node = Node(self, parent_idx, level)
        for key, value in kwargs.items(): node.set(key, value)
        index = self.__add(parent_idx, level, node)
        if node.has_parent():
            node.get_parent().add_child(index)
        return index

    """
    添加树的节点
    :param data
    格式要求 字典列表 [dict(),dict()]
    data的长度 = total_level
    """
    def add(self, data: list[dict]):
        # 第一次节点是没有父节点的，所以首次把 parent 设置为 None
        parent = None
        for level in range(self.total_level):
            parent = self.__create_node(parent, level, **data[level])

    def clean(self):
        self.nodes: list[list[Node]] = []
        self.total_level = -1

    def index(self, level, i) -> Node:
        return self.get_level_nodes(level)[i]

    """
    指定level 
    指定索引搜索满足props条件的值
    :return 返回一个值, 如没有则返回None
    """

    def find(self, level, xrange: set, **props) -> Node:
        result = list(filter(lambda i: self.nodes[level][i].satisfy(**props), xrange))
        if not result: return None
        return self.nodes[level][result[0]]

    """
    指定level
    搜索满足props条件的值
    :return 返回所有满足条件的
    """

    def search(self, level, **props):
        return list(filter(lambda node: node.satisfy(**props), self.nodes[level]))

    def get_level_nodes(self, level) -> list[Node]:
        return self.nodes[level]

    def __add(self, parent_idx, level, node) -> int:
        level_nodes = self.nodes[level]
        if parent_idx is None:
            # level = 0 的层级节点
            # 因为level = 0 层级的节点，没有父节点，所以直接判断是否已存在相同节点，如果存在直接返回索引值
            for i in range(len(level_nodes)):
                if level_nodes[i].eql(node): return i
        else:
            # level != 0 的层级节点
            # 因为是有父节点的，所以判断有相同父节点的兄弟节点中是否已存在相同节点，如果存在直接返回索引值
            for i in self.nodes[level - 1][parent_idx].children_idx:
                if level_nodes[i].eql(node): return i
        # 节点为新节点,加入对应层级的节点列表，并返回索引
        self.nodes[level].append(node)
        return len(level_nodes) - 1
    """
    :parameter new_total_level
    默认是不改变树的层级数,如果要改变则需要传递此参数
    """
    def refresh(self, data: list[list[dict]], new_total_level=None):
        total_level = new_total_level if new_total_level else self.total_level
        self.clean()
        self.total_level = total_level
        self.nodes = self.new_node_list()
        for e in data: self.add(e)

    @classmethod
    def factory(cls, total_level, data: list[dict] = None):
        tree = Tree(total_level=total_level)
        if data is not None: tree.add(data)
        return tree



from cjen.nene.collections.tree import Tree


def list_eql(arr1: list, arr2: list, strict: bool = False):
    """
    判断 两个列表是否相同
    strict = True: 严格模式
        a=[1,2] , b=[1,2] ab相同
        a=[1,2] , b=[2,1] ab不相同
    strict = False: 非严格模式
        a=[1,2] , b=[1,2] ab相同
        a=[1,2] , b=[2,1] ab相同
    exception list_eql([[1,2], [3,4]], [[2,1], [4,3]]) 判断为 False
    """
    if strict: return arr1 == arr2
    if len(arr1) != len(arr2): return False
    for e in arr1:
        if e not in arr2: return False
    for e in arr2:
        if e not in arr1: return False
    return True


def make_tree(total_level, data: list[dict] = None):
    return Tree.factory(total_level, data)
